<?php

/**
 * Plugin Name: Pay 7
 * Plugin URI: https://pay7.maticz.in/
 * Author Name: Aswinkumar
 * Author URI: https://maticz.com/
 * Description: This plugin allows for local content payment systems.
 * Version: 4.3
 * License: GPLv2 or later
 * License URL: http://www.gnu.org/licenses/gpl-2.0.html
 * Requires Plugins: woocommerce
 * text-domain: pay-7-for-woocommerce
*/ 
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) return;

add_action( 'plugins_loaded', 'PAY7FOWO__payment_init', 11 );

function PAY7FOWO__payment_init() {
    if( class_exists( 'WC_Payment_Gateway' ) ) {
        #[AllowDynamicProperties] 
        class PAY7FOWO_pay_Gateway extends WC_Payment_Gateway {

            
            public function __construct() {
                $this->id   = 'noob_payment';
              //  $this->icon = apply_filters( 'woocommerce_noob_icon', plugins_url('/assets/icon.png', __FILE__ ) );
                $this->has_fields = false;
                $this->method_title = __( 'Pay7 Payment', 'pay-7-for-woocommerce');
                $this->method_description = __( 'Pay7 payment systems.', 'pay-7-for-woocommerce');
                $this->ipn_url   = add_query_arg( 'wc-api', 'PAY7FOWO_pay_Gateway', home_url( '/' ) );

                $this->title = $this->get_option( 'title' );
                $this->description = $this->get_option( 'description' );

                $this->init_form_fields();
                $this->init_settings();

                	// Define user set variables
                $this->merchant_id 			= $this->get_option( 'merchant_id' );
                $this->ipn_secret   = $this->get_option( 'ipn_secret' );
                $this->send_shipping	= $this->get_option( 'send_shipping' );
                $this->debug_email			= $this->get_option( 'debug_email' );
                $this->allow_zero_confirm = $this->get_option( 'allow_zero_confirm' ) == 'yes' ? true : false;
                $this->form_submission_method = $this->get_option( 'form_submission_method' ) == 'yes' ? true : false;
                $this->invoice_prefix	= $this->get_option( 'invoice_prefix', 'WC-' );
                $this->simple_total = $this->get_option( 'simple_total' ) == 'yes' ? true : false;

               // add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

                // Logs
                $this->log = new WC_Logger();

                // Actions
                add_action( 'woocommerce_receipt_noobpayments', array( $this, 'receipt_page' ) );
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

                // Payment listener/API hook
                add_action( 'woocommerce_api_wc_gateway_noobpayments', array( $this, 'check_ipn_response' ) );

                if ( !$this->is_valid_for_use() ) $this->enabled = false;
                
            }

             /**
     * Check if this gateway is enabled and available in the user's country
     *
     * @access public
     * @return bool
     */
    function is_valid_for_use() {
        //if ( ! in_array( get_woocommerce_currency(), apply_filters( 'woocommerce_noobpayments_supported_currencies', array( 'AUD', 'CAD', 'USD', 'EUR', 'JPY', 'GBP', 'CZK', 'BTC', 'LTC' ) ) ) ) return false;
        // ^- instead of trying to maintain this list just let it always work
        return true;
    }

	/**
	 * Admin Panel Options
	 * - Options for bits like 'title' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {

		?>
		<h3><?php esc_html_e( 'NoobPayments', 'pay-7-for-woocommerce' ); ?></h3>
		<p><?php esc_html_e( 'Completes checkout via NoobPayments', 'pay-7-for-woocommerce' ); ?></p>

    	<?php if ( $this->is_valid_for_use() ) : ?>

			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->

		<?php else : ?>
            <div class="inline error"><p><strong><?php esc_html_e( 'Gateway Disabled', 'pay-7-for-woocommerce' ); ?></strong>: <?php esc_html_e( 'NoobPayments does not support your store currency.', 'pay-7-for-woocommerce' ); ?></p></div>
		<?php
			endif;
	}


    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {

    	$this->form_fields = array(
			'enabled' => array(
							'title' => __( 'Enable/Disable', 'pay-7-for-woocommerce' ),
							'type' => 'checkbox',
							'label' => __( 'Enable NoobPayments', 'pay-7-for-woocommerce' ),
							'default' => 'yes'
						),
			'title' => array(
							'title' => __( 'Title', 'pay-7-for-woocommerce' ),
							'type' => 'text',
							'description' => __( 'This controls the title which the user sees during checkout.', 'pay-7-for-woocommerce' ),
							'default' => __( 'NoobPayments', 'pay-7-for-woocommerce' ),
							'desc_tip'      => true,
						),
			'description' => array(
							'title' => __( 'Description', 'pay-7-for-woocommerce' ),
							'type' => 'textarea',
							'description' => __( 'This controls the description which the user sees during checkout.', 'pay-7-for-woocommerce' ),
							'default' => __( 'Pay with Bitcoin, Litecoin, or other altcoins via Pay7', 'pay-7-for-woocommerce' )
						),
			'merchant_id' => array(
							'title' => __( 'Wallet ID', 'pay-7-for-woocommerce' ),
							'type' 			=> 'text',
							'description' => __( 'Please enter your Pay7 Wallet ID.', 'pay-7-for-woocommerce' ),
							'default' => '',
						),
			'ipn_secret' => array(
							'title' => __( 'IPN Secret', 'pay-7-for-woocommerce' ),
							'type' 			=> 'text',
							'description' => __( 'Please enter your Pay7 IPN Secret.', 'pay-7-for-woocommerce' ),
							'default' => '',
						),
			'simple_total' => array(
							'title' => __( 'Compatibility Mode', 'pay-7-for-woocommerce' ),
							'type' => 'checkbox',
							'label' => __( "This may be needed for compatibility with certain addons if the order total isn't correct.", 'pay-7-for-woocommerce' ),
							'default' => ''
						),
			'send_shipping' => array(
							'title' => __( 'Collect Shipping Info?', 'pay-7-for-woocommerce' ),
							'type' => 'checkbox',
							'label' => __( 'Enable Shipping Information on Checkout page', 'pay-7-for-woocommerce' ),
							'default' => 'yes'
						),
			'allow_zero_confirm' => array(
							'title' => __( 'Enable 1st-confirm payments?', 'pay-7-for-woocommerce' ),
							'type' => 'checkbox',
							'label' => __( '* WARNING * If this is selected orders will be marked as paid as soon as your buyer\'s payment is detected, but before it is fully confirmed. This can be dangerous if the payment never confirms and is only recommended for digital downloads.', 'pay-7-for-woocommerce' ),
							'default' => ''
						),
			'invoice_prefix' => array(
							'title' => __( 'Invoice Prefix', 'pay-7-for-woocommerce' ),
							'type' => 'text',
							'description' => __( 'Please enter a prefix for your invoice numbers. If you use your Pay7 account for multiple stores ensure this prefix is unique.', 'pay-7-for-woocommerce' ),
							'default' => 'WC-',
							'desc_tip'      => true,
						),
			'testing' => array(
							'title' => __( 'Gateway Testing', 'pay-7-for-woocommerce' ),
							'type' => 'title',
							'description' => '',
						),
			'debug_email' => array(
							'title' => __( 'Debug Email', 'pay-7-for-woocommerce' ),
							'type' => 'email',
							'default' => '',
							'description' => __( 'Send copies of invalid IPNs to this email address.', 'pay-7-for-woocommerce' ),
						)
			);

    }


        /**
         * Get NoobPayments Args
         *
         * @access public
         * @param mixed $order
         * @return array
         */
        function get_noobpayments_args( $order ) {
            global $woocommerce;

            $order_id = $order->get_id();

            if ( in_array( $order->get_billing_country(), array( 'US','CA' ) ) ) {
                $order->set_billing_phone(str_replace( array( '( ', '-', ' ', ' )', '.' ), '', $order->get_billing_phone() ));
            }

            // NoobPayments Args
            $noobpayments_args = array(
                    'cmd' 					=> '_pay_auto',
                    'merchantId' 				=> $this->merchant_id,
                    'allow_extra' 				=> 0,
                    'ipn_secret'				=>$this->ipn_secret,
                    // Get the currency from the order, not the active currency
                    'currency' 		=> $order->get_currency(),
                    'reset' 				=> 1,
                    'success_url' 				=> $this->get_return_url( $order ),
                    'cancel_url'			=> esc_url_raw($order->get_cancel_order_url_raw()),

                    // Order key + ID
                    'invoice'				=> $this->invoice_prefix . $order->get_order_number(),
                    'custom' 				=> serialize( array( $order->get_id(), $order->get_order_key() ) ),

                    // IPN
                    'ipn_url'			=> $this->ipn_url,

                    // Billing Address info
                    'first_name'			=> $order->get_billing_first_name(),
                    'last_name'				=> $order->get_billing_last_name(),
                    'email'					=> $order->get_billing_email(),
            );

            if ($this->send_shipping == 'yes') {
                $noobpayments_args = array_merge($noobpayments_args, array(
                    'want_shipping' => 1,
                    'company'					=> $order->get_billing_company(),
                    'address1'				=> $order->get_billing_address_1(),
                    'address2'				=> $order->get_billing_address_2(),
                    'city'					=> $order->get_billing_city(),
                    'state'					=> $order->get_billing_state(),
                    'zip'					=> $order->get_billing_postcode(),
                    'country'				=> $order->get_billing_country(),
                    'phone'					=> $order->get_billing_phone(),
                ));
            } else {
                $noobpayments_args['want_shipping'] = 0;
            }

            if ($this->simple_total) {
                $noobpayments_args['item_name'] 	= sprintf( __( 'Order %s' , 'pay-7-for-woocommerce'), $order->get_order_number() );
                $noobpayments_args['quantity'] 		= 1;
                $noobpayments_args['amountf'] 		= number_format( $order->get_total(), 8, '.', '' );
                $noobpayments_args['taxf'] 				= 0.00;
                $noobpayments_args['shippingf']		= 0.00;
            } else if ( wc_tax_enabled() && wc_prices_include_tax() ) {
                $noobpayments_args['item_name'] 	= sprintf( __( 'Order %s' , 'pay-7-for-woocommerce'), $order->get_order_number() );
                $noobpayments_args['quantity'] 		= 1;
                $noobpayments_args['amountf'] 		= number_format( $order->get_total() - $order->get_total_shipping() - $order->get_shipping_tax(), 8, '.', '' );
                $noobpayments_args['shippingf']		= number_format( $order->get_total_shipping() + $order->get_shipping_tax() , 8, '.', '' );
                $noobpayments_args['taxf'] 				= 0.00;
            } else {
                $noobpayments_args['item_name'] 	= sprintf( __( 'Order %s' , 'pay-7-for-woocommerce'), $order->get_order_number() );
                $noobpayments_args['quantity'] 		= 1;
                $noobpayments_args['amountf'] 		= number_format( $order->get_total() - $order->get_total_shipping() - $order->get_total_tax(), 8, '.', '' );
                $noobpayments_args['shippingf']		= number_format( $order->get_total_shipping(), 8, '.', '' );
                $noobpayments_args['taxf']				= number_format( $order->get_total_tax(), 8, '.', '' );
            }

            $noobpayments_args = apply_filters( 'PAY7FOWO_noobpayments_args', $noobpayments_args );

            return $noobpayments_args;
        }


        /**
         * Generate the noobpayments button link
         *
         * @access public
         * @param mixed $order_id
         * @return string
         */
        function generate_noobpayments_url($order) {
            global $woocommerce;

            if ( $order->get_status() != 'completed' && get_post_meta($order->get_id(), 'CoinPayments payment complete', true ) != 'Yes' ) {
                //$order->update_status('on-hold', 'Customer is being redirected to CoinPayments...');
                $order->update_status('pending', 'Customer is being redirected to CoinPayments...');
            }

$noobpayments_adr = "https://pay7.maticz.in/paymentgateway?";
            $noobpayments_args = $this->get_noobpayments_args( $order );
            $noobpayments_adr .= http_build_query( $noobpayments_args, '', '&' );
            return $noobpayments_adr;
        }


        /**
         * Process the payment and return the result
         *
         * @access public
         * @param int $order_id
         * @return array
         */
        function process_payment( $order_id ) {

            $order          = wc_get_order( $order_id );

            return array(
                    'result' 	=> 'success',
                    'redirect'	=> $this->generate_noobpayments_url($order),
            );

        }


        /**
         * Output for the order received page.
         *
         * @access public
         * @return void
         */
        function receipt_page( $order ) {
    // Ensure all text content is escaped before output
    echo '<p>' . esc_html__( 'Thank you for your order, please click the button below to pay with Pay7.', 'pay-7-for-woocommerce' ) . '</p>';

    // Ensure the form output is escaped if necessary (depending on what generate_noobpayments_form() returns)
    echo $this->generate_noobpayments_form( $order );
}


        /**
         * Check NoobPayments IPN validity
         **/
         function check_ipn_request_is_valid() {
            global $woocommerce;
        
            $order = false;
            $error_msg = "Unknown error";
            $auth_ok = false;
        
            // Check if IPN mode is 'hmac' (which means HMAC signature verification is used)
            if (isset($_POST['ipn_mode']) && $_POST['ipn_mode'] == 'hmac') {
                // Validate if HMAC signature is present
                if (isset($_SERVER['HTTP_HMAC']) && !empty($_SERVER['HTTP_HMAC'])) {
                    // Get the raw POST data
                    $request = file_get_contents('php://input');
        
                    if ($request !== false && !empty($request)) {
                        // Check if the merchant ID is correct
                        if (isset($_POST['merchant']) && $_POST['merchant'] == trim($this->merchant_id)) {
                            // Create the HMAC hash using the raw POST data and IPN secret
                            $hmac = hash_hmac("sha512", $request, trim($this->ipn_secret));
        
                            // Compare the HMAC hash with the signature received in the request
                            if ($hmac == $_SERVER['HTTP_HMAC']) {
                                $auth_ok = true;
                            } else {
                                $error_msg = 'HMAC signature does not match';
                            }
                        } else {
                            $error_msg = 'No or incorrect Merchant ID passed';
                        }
                    } else {
                        $error_msg = 'Error reading POST data';
                    }
                } else {
                    $error_msg = 'No HMAC signature sent.';
                }
            } else {
                $error_msg = "Unknown IPN verification method.";
            }
        
            // Continue processing only if HMAC authentication is successful
            if ($auth_ok) {
                // Ensure invoice and custom fields are provided
                if (!empty($_POST['invoice']) && !empty($_POST['custom'])) {
                    // Retrieve the order details using the invoice and custom fields
                    $order = $this->get_noobpayments_order($_POST);
                }
        
                // Check if the order was retrieved successfully
                if ($order !== false) {
                    // Process valid IPN based on ipn_type
                    if ($_POST['ipn_type'] == "button" || $_POST['ipn_type'] == "simple") {
                        // Ensure merchant ID matches
                        if ($_POST['merchant'] == $this->merchant_id) {
                            // Check if currency matches the order's currency
                            if ($_POST['currency1'] == $order->get_currency()) {
                                // Ensure the amount received is equal or greater than the order total
                                if ($_POST['amount1'] >= $order->get_total()) {
                                    // Successful validation
                                    print "IPN check OK\n";
                                    return true;
                                } else {
                                    $error_msg = "Amount received is less than the total!";
                                }
                            } else {
                                $error_msg = "Original currency doesn't match!";
                            }
                        } else {
                            $error_msg = "Merchant ID doesn't match!";
                        }
                    } else {
                        $error_msg = "ipn_type != button or simple";
                    }
                } else {
                    $error_msg = "Could not find order info for order: ".$_POST['invoice'];
                }
            }
        
            // Prepare detailed report on failure
            $report = "Error Message: ".$error_msg."\n\n";
        
            $report .= "POST Fields\n\n";
            foreach ($_POST as $key => $value) {
                $report .= $key.'='.$value."\n";
            }
        
            // Update order status to 'on-hold' in case of error
            if ($order) {
                $order->update_status('on-hold', sprintf( __('NoobPayments IPN Error: %s', 'pay-7-for-woocommerce'), $error_msg));
            }
        
            // Send the report to the admin and debug email if set
            if (!empty($this->debug_email)) {
                mail($this->debug_email, "NoobPayments Invalid IPN", $report);
            }
            mail(get_option('admin_email'), sprintf(__('NoobPayments Invalid IPN', 'pay-7-for-woocommerce'), $error_msg), $report);
        
            // Exit with error message
            die('IPN Error: '.$error_msg);
            return false;
        }

        /**
         * Successful Payment!
         *
         * @access public
         * @param array $posted
         * @return void
         */
        function successful_request( $posted ) {
            global $woocommerce;
        
            // Sanitize the input data (stripslashes, sanitize, etc.)
            $posted = stripslashes_deep( $posted );
        
            // Validate that the invoice and custom are set and numeric
            if ( ! empty( $_POST['invoice'] ) && ! empty( $_POST['custom'] ) ) {
                $invoice = sanitize_text_field( $_POST['invoice'] ); // sanitize invoice
                $custom = sanitize_text_field( $_POST['custom'] );   // sanitize custom
        
                // Retrieve the order based on custom logic
                $order = $this->get_noobpayments_order( $posted );
                
                if ( $order === FALSE ) {
                    die( "IPN Error: Could not find order info for order: " . esc_html( $invoice ) );
                }
        
                // Log the status, sanitize the status text to prevent XSS
                $this->log->add( 'noobpayments', 'Order #' . esc_html( $order->get_id() ) . ' payment status: ' . sanitize_text_field( $posted['status_text'] ) );
                $order->add_order_note( 'NoobPayments Payment Status: ' . sanitize_text_field( $posted['status_text'] ) );
        
                // Ensure the order status isn't 'completed' and check the custom payment meta
                if ( $order->get_status() != 'completed' && get_post_meta( $order->get_id(), 'CoinPayments payment complete', true ) != 'Yes' ) {
                    
                    // Prevent empty or unsafe transaction IDs, names, and email
                    if ( ! empty( $posted['txn_id'] ) ) {
                        $txn_id = sanitize_text_field( $posted['txn_id'] );
                        update_post_meta( $order->get_id(), 'Transaction ID', $txn_id );
                    }
        
                    if ( ! empty( $posted['first_name'] ) ) {
                        $first_name = sanitize_text_field( $posted['first_name'] );
                        update_post_meta( $order->get_id(), 'Payer first name', $first_name );
                    }
        
                    if ( ! empty( $posted['last_name'] ) ) {
                        $last_name = sanitize_text_field( $posted['last_name'] );
                        update_post_meta( $order->get_id(), 'Payer last name', $last_name );
                    }
        
                    if ( ! empty( $posted['email'] ) ) {
                        $email = sanitize_email( $posted['email'] ); // sanitize email
                        if ( is_email( $email ) ) {
                            update_post_meta( $order->get_id(), 'Payer email', $email );
                        } else {
                            die( "IPN Error: Invalid email address" );
                        }
                    }
                    // Validate payment status and confirm the payment status
                    if ( $posted['status'] >= 100 || $posted['status'] == 2 || ( $this->allow_zero_confirm && $posted['status'] >= 0 && $posted['received_confirms'] > 0 && $posted['received_amount'] >= $posted['amount2'] ) ) {
                        print "Marking complete\n";
                        update_post_meta( $order->get_id(), 'CoinPayments payment complete', 'Yes' );
                        $order->payment_complete();
                    } elseif ( $posted['status'] < 0 ) {
                        print "Marking cancelled\n";
                        $order->update_status( 'cancelled', 'NoobPayments Payment cancelled/timed out: ' . sanitize_text_field( $posted['status_text'] ) );
                        mail( get_option( 'admin_email' ), sprintf( __( 'Payment for order %s cancelled/timed out', 'pay-7-for-woocommerce' ), $order->get_order_number() ), sanitize_text_field( $posted['status_text'] ) );
                    } else {
                        print "Marking pending\n";
                        $order->update_status( 'pending', 'NoobPayments Payment pending: ' . sanitize_text_field( $posted['status_text'] ) );
                    }
                }
                die( "IPN OK" );
            }
        }

        

        /**
         * Check for CoinPayments IPN Response
         *
         * @access public
         * @return void
         */
        function check_ipn_response() {

            @ob_clean();

            if ( ! empty( $_POST ) && $this->check_ipn_request_is_valid() ) {
                $this->successful_request($_POST);
            } else {
                wp_die( "NoobPayments IPN Request Failure" );
            }
        }

        /**
         * get_noobpayments_order function.
         *
         * @access public
         * @param mixed $posted
         * @return void
         */
        function get_noobpayments_order( $posted ) {
            $custom = maybe_unserialize( stripslashes_deep($posted['custom']) );

            // Backwards comp for IPN requests
            if ( is_numeric( $custom ) ) {
                $order_id = (int) $custom;
                $order_key = $posted['invoice'];
            } elseif( is_string( $custom ) ) {
                $order_id = (int) str_replace( $this->invoice_prefix, '', $custom );
                $order_key = $custom;
            } else {
                list( $order_id, $order_key ) = $custom;
            }

            $order = wc_get_order( $order_id );

            if ($order === FALSE) {
                // We have an invalid $order_id, probably because invoice_prefix has changed
                $order_id 	= wc_get_order_id_by_order_key( $order_key );
                $order 		= wc_get_order( $order_id );
            }

            // Validate key
            if ($order === FALSE || $order->get_order_key() !== $order_key ) {
                return FALSE;
            }

            return $order;
        }
        }
    }
}

add_filter( 'woocommerce_payment_gateways', 'PAY7FOWO_payment_gateway');

function PAY7FOWO_payment_gateway( $gateways ) {
    $gateways[] = 'PAY7FOWO_pay_Gateway';
    return $gateways;
}

